"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const swtc_utils_1 = require("swtc-utils");
const index_1 = require("../store/index");
const rest_1 = require("../web/rest");
function currencyFlatten(options) {
    return Object.values(options)
        .filter(e => e)
        .join("+");
}
exports.currencyFlatten = currencyFlatten;
function currencyUnFlatten(currency) {
    const pairs = currency.split("+");
    return { currency: pairs[0], issuer: pairs[1] || "" };
}
exports.currencyUnFlatten = currencyUnFlatten;
async function generateWallet(ctx) {
    ctx.rest(index_1.Wallet.generate());
}
exports.generateWallet = generateWallet;
async function getAccountInfo(ctx) {
    const data = await index_1.state.remote.value
        .requestAccountInfo({ account: ctx.params.address })
        .submitPromise();
    ctx.rest(data);
}
exports.getAccountInfo = getAccountInfo;
async function getAccountTrusts(ctx) {
    const data = await index_1.state.remote.value
        .requestAccountRelations({ type: "trust", account: ctx.params.address })
        .submitPromise();
    ctx.rest(data);
}
exports.getAccountTrusts = getAccountTrusts;
async function getAccountAuthorizes(ctx) {
    const data = await index_1.state.remote.value
        .requestAccountRelations({ type: "authorize", account: ctx.params.address })
        .submitPromise();
    ctx.rest(data);
}
exports.getAccountAuthorizes = getAccountAuthorizes;
async function getAccountFreezes(ctx) {
    const data = await index_1.state.remote.value
        .requestAccountRelations({ type: "freeze", account: ctx.params.address })
        .submitPromise();
    ctx.rest(data);
}
exports.getAccountFreezes = getAccountFreezes;
async function getAccountBalances(ctx) {
    const p_info = index_1.state.remote.value
        .requestAccountInfo({ account: ctx.params.address })
        .submitPromise();
    const p_trust = index_1.state.remote.value
        .requestAccountRelations({ type: "trust", account: ctx.params.address })
        .submitPromise();
    const p_freeze = index_1.state.remote.value
        .requestAccountRelations({ type: "freeze", account: ctx.params.address })
        .submitPromise();
    const data = await Promise.all([p_info, p_trust, p_freeze]);
    ctx.rest({ info: data[0], trusts: data[1], freezes: data[2] });
}
exports.getAccountBalances = getAccountBalances;
async function getAccountPayment(ctx) {
    const address = ctx.params.address;
    const data = await index_1.state.remote.value
        .requestTx({ hash: ctx.params.id })
        .submitPromise();
    if (data.TransactionType &&
        data.TransactionType === "Payment" &&
        ((data.Account && data.Account === address) ||
            (data.Destination && data.Destination === address))) {
        ctx.rest(swtc_utils_1.utils.processTx(data, address));
    }
    else {
        throw new rest_1.APIError("api:param", `does not exist on server ${JSON.stringify(ctx.params)}`);
    }
}
exports.getAccountPayment = getAccountPayment;
async function getAccountPayments(ctx) {
    const data = await index_1.state.remote.value
        .requestAccountTx({ account: ctx.params.address, limit: index_1.state.LIMIT.value })
        .submitPromise();
    const payments = data.transactions.filter(e => (e.type && e.type === "received") || e.type === "sent");
    delete data.transactions;
    data.payments = payments;
    ctx.rest(data);
}
exports.getAccountPayments = getAccountPayments;
async function getAccountTransaction(ctx) {
    const address = ctx.params.address;
    const data = await index_1.state.remote.value
        .requestTx({ hash: ctx.params.id })
        .submitPromise();
    if ((data.Account && data.Account === address) ||
        (data.Destination && data.Destination === address)) {
        ctx.rest(swtc_utils_1.utils.processTx(data, address));
    }
    else {
        if (data.meta &&
            data.meta.AffectedNodes &&
            data.meta.AffectedNodes.filter(e => e.ModifiedNode &&
                e.ModifiedNode.FinalFields &&
                e.ModifiedNode.FinalFields.Account === ctx.params.address).length === 1) {
            // effects
            ctx.rest(swtc_utils_1.utils.processTx(data, address));
        }
        else {
            // ?? what about relations ??
            throw new rest_1.APIError("api:param", `does not exist on server ${JSON.stringify(ctx.params)}`);
        }
    }
}
exports.getAccountTransaction = getAccountTransaction;
async function getAccountTransactions(ctx) {
    const data = await index_1.state.remote.value
        .requestAccountTx({ account: ctx.params.address, limit: index_1.state.LIMIT.value })
        .submitPromise();
    ctx.rest(data);
}
exports.getAccountTransactions = getAccountTransactions;
async function getAccountOrder(ctx) {
    const address = ctx.params.address;
    const hash = ctx.params.hash;
    const data = await index_1.state.remote.value.requestTx({ hash }).submitPromise();
    if (data.TransactionType &&
        data.TransactionType === "OfferCreate" &&
        data.Account &&
        data.Account === address) {
        ctx.rest(swtc_utils_1.utils.processTx(data, address));
    }
    else {
        throw new rest_1.APIError("api:param", `does not exist on server ${JSON.stringify(ctx.params)}`);
    }
}
exports.getAccountOrder = getAccountOrder;
async function getAccountOrders(ctx) {
    const account = ctx.params.address;
    const base = ctx.params.base;
    const counter = ctx.params.counter;
    const data = await index_1.state.remote.value
        .requestAccountOffers({
        account,
        base,
        counter,
        ledger: "closed",
        limit: index_1.state.LIMIT.value
    })
        .submitPromise();
    ctx.rest(data);
}
exports.getAccountOrders = getAccountOrders;
async function getOrderBook(ctx) {
    const gets = currencyUnFlatten(ctx.params.base);
    const pays = currencyUnFlatten(ctx.params.counter);
    const data = await index_1.state.remote.value
        .requestOrderBook({ gets, pays, limit: index_1.state.LIMIT.value })
        .submitPromise();
    ctx.rest(data);
    // const p_asks = state.remote.value
    //  .requestOrderBook({ gets: pays, pays: gets, limit: state.LIMIT.value })
    //  .submitPromise()
    // const data = await Promise.all([p_bids, p_asks])
    // ctx.rest({ bids: data[0], asks: data[1] })
}
exports.getOrderBook = getOrderBook;
async function getOrderBookBids(ctx) {
    const gets = currencyUnFlatten(ctx.params.base);
    const pays = currencyUnFlatten(ctx.params.counter);
    const data = await index_1.state.remote.value
        .requestOrderBook({ gets, pays, limit: index_1.state.LIMIT.value })
        .submitPromise();
    ctx.rest(data);
}
exports.getOrderBookBids = getOrderBookBids;
async function getOrderBookAsks(ctx) {
    const gets = currencyUnFlatten(ctx.params.base);
    const pays = currencyUnFlatten(ctx.params.counter);
    const data = await index_1.state.remote.value
        .requestOrderBook({ gets: pays, pays: gets, limit: index_1.state.LIMIT.value })
        .submitPromise();
    ctx.rest(data);
}
exports.getOrderBookAsks = getOrderBookAsks;
async function getTransaction(ctx) {
    const hash = ctx.params.hash;
    const data = await index_1.state.remote.value.requestTx({ hash }).submitPromise();
    if (data.date) {
        ctx.rest(data);
    }
    else {
        // ?? what about relations ??
        throw new rest_1.APIError("api:param", `does not exist on server ${JSON.stringify(ctx.params)}`);
    }
}
exports.getTransaction = getTransaction;
async function getLedgerClosed(ctx) {
    const { ledger_hash, ledger_index } = index_1.state.ledger.value;
    ctx.rest({ ledger_hash, ledger_index });
}
exports.getLedgerClosed = getLedgerClosed;
async function getLedgerHash(ctx) {
    const hash = ctx.params.hash;
    const transactions = true;
    const data = await index_1.state.remote.value
        .requestLedger({ hash, transactions })
        .submitPromise();
    if (data.ledger_index) {
        ctx.rest(data);
    }
    else {
        // ?? what about relations ??
        throw new rest_1.APIError("api:param", `does not exist on server ${JSON.stringify(ctx.params)}`);
    }
}
exports.getLedgerHash = getLedgerHash;
async function getLedgerIndex(ctx) {
    const index = ctx.params.index;
    const transactions = true;
    const data = await index_1.state.remote.value
        .requestLedger({ index, transactions })
        .submitPromise();
    if (data.ledger_index) {
        ctx.rest(data);
    }
    else {
        // ?? what about relations ??
        throw new rest_1.APIError("api:param", `does not exist on server ${JSON.stringify(ctx.params)}`);
    }
}
exports.getLedgerIndex = getLedgerIndex;
async function postBlob(ctx) {
    const data = ctx.request.body;
    const tx = index_1.state.remote.value.buildSignTx(data);
    console.log(tx.tx_json);
    ctx.rest(await tx.submitPromise());
}
exports.postBlob = postBlob;
async function postJsonMultisign(ctx) {
    const data = ctx.request.body;
    console.log(data);
    const tx = index_1.state.remote.value.buildMultisignedTx(data);
    tx.multiSigned();
    console.log(tx.tx_json);
    ctx.rest(await tx.submitPromise());
}
exports.postJsonMultisign = postJsonMultisign;
//# sourceMappingURL=v1.js.map